//---------------------------------------------------------------------
//  Copyright (C) Microsoft Corporation.  All rights reserved.
// 
// THIS CODE AND INFORMATION ARE PROVIDED AS IS WITHOUT WARRANTY OF ANY
// KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
// PARTICULAR PURPOSE.
//
//---------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Design;
using System.Windows.Forms;
using System.Windows.Forms.Design;
using System.Windows.Forms.Design.Behavior;

namespace DesignSurfaceExtenderLibrary.Extensions
{
    /// <summary>
    /// Provides an abstract base implementation for <see cref="Glyph"/> types
    /// which represent the bounds of a control.  
    /// </summary>
    /// <remarks>This implementation assumes at least one <see cref="BoundsGlyph"/> is
    /// assigned on a per-control basis.
    /// </remarks>
    internal abstract class BoundsGlyph : ExtensionGlyph
    {
        protected readonly int hitTestWidth = 4;

        protected Rectangle boundsLeftRect;
        protected Rectangle boundsTopRect;
        protected Rectangle boundsRightRect;
        protected Rectangle boundsBottomRect;

        protected string propertyName;


        internal BoundsGlyph(
            Control c,
            BehaviorService behaviorService,
            IComponentChangeService changeService,
            ISelectionService selectionService,
            IReferenceService referenceService,
            IServiceProvider serviceProvider,
            BoundsBehavior boundsBehavior,
            Adorner adorner)
            : base( 
            c,
                behaviorService,
                changeService,
                selectionService,
                referenceService,
                serviceProvider,
                boundsBehavior,
                adorner )
        {   
        }

        protected override ReadOnlyCollection<string> PopulateDependentProperties()
        {
            List<string> properties = new List<string>();

            properties.Add( "Margin" );
            properties.Add( "Padding" );
            properties.Add( "Size" );
            properties.Add( "Height" );
            properties.Add( "Width" );
            properties.Add( "Location" );

            return new ReadOnlyCollection<string>( properties );
        }

        public override Cursor GetHitTest( Point p )
        {
            Cursor c = null;

            if( this.boundsLeftRect.Contains( p ) ||
                this.boundsRightRect.Contains( p ) )
            {
                c = Cursors.SizeWE;
            }
            else if( this.boundsTopRect.Contains( p ) ||
                this.boundsBottomRect.Contains( p ) )
            {
                c = Cursors.SizeNS;
            }

            return c;
        }

        // This method renders the glyph as a simple focus rectangle.
        public override void Paint( PaintEventArgs e )
        {
            ControlPaint.DrawFocusRectangle(
                    e.Graphics,
                    this.Bounds );
        }

        internal EdgeSelection SelectedEdge( Point loc )
        {
            EdgeSelection selectedEdge = EdgeSelection.None;

            if( this.GetHitTest( loc ) != null )
            {
                if( this.boundsLeftRect.Contains( loc ) )
                {
                    selectedEdge = EdgeSelection.Left;
                }
                else if( this.boundsTopRect.Contains( loc ) )
                {
                    selectedEdge = EdgeSelection.Top;
                }
                else if( this.boundsRightRect.Contains( loc ) )
                {
                    selectedEdge = EdgeSelection.Right;
                }
                else if( this.boundsBottomRect.Contains( loc ) )
                {
                    selectedEdge = EdgeSelection.Bottom;
                }
            }

            return selectedEdge;
        }
    }
}
